
set frame SECURITY
set title "Security option interface"
set icon Security

set rulesdata {}

#    current system passwords
foreach i {0 1 2 3 4 5 6 7 8 9} {set Syspassword($i) ""}
#    user supplied passwords
foreach i {0 1 2 3 4 5 6 7 8 9} {set Userpassword($i) ""}


proc initialise {} {
   global env rulesdata Syspassword

#    read in the security rules file
   set rules [file join $env(MIDASSECURITY) rules]
   if {![file exists $rules]} then {
#    no security rules file - disable security option
      set env(MIDASSECURITY) "disabled"
      puts stdout "Security option disabled"
      return
   }
   set f [open $rules r]
   set rulesdata [split [read $f] \n]
   close $f

#    read in the password file
   set passwords [file join $env(MIDASSECURITY) passwords]
   if {![file exists $passwords]} then {
#    no security passwords file - disable security option
      set env(MIDASSECURITY) "disabled"
      puts stdout "Security option disabled"
      return
   }
   set f [open $passwords r]
   set passworddata [split [split [read $f] \n] ,]
   close $f

   foreach i {1 2 3 4 5 6 7 8 9} {set Syspassword($i) ""}
   set i 1
   foreach item $passworddata {
      set Syspassword($i) [first $item]
      incr i
   }

   puts stdout "Security option enabled"
}

#
#    request has the form  [task name],action    OR
#                          [task name].window,action    WHERE
#     action is  c (create), r (read} or w (write)
#
proc validate {request} {
   global env rulesdata Syspassword Userpassword

   if {$env(MIDASSECURITY) == "disabled"} then {return 1}

   set r [split $request ,]
   set task [first $r]
   set action [last $r]

#    find the access level for this request

   set item ""
   foreach item $rulesdata {
      if {[string match [first $item] $task]} then {break}
   }

   if {$item == ""} then {
#    no information for this request - default is to validate
     set reply 1

   } else {
     switch $action {
        c  {
           set l [second $item]
           if {$l == "f"} then {set l 0}
           if {$l == "p"} then {set l 1}
           if {$Syspassword($l) == $Userpassword($l)} then {set reply 1} else {set reply 0}
        }
        r  {
           set l [first [split [third $item] ""]]
           if {$l == "f"} then {set l 0}
           if {$l == "p"} then {set l 1}
           if {$Syspassword($l) == $Userpassword($l)} then {set reply 1} else {set reply 0}
        }
        w  {
           set l [second [split [third $item] ""]]
           if {$l == "f"} then {set l 0}
           if {$l == "p"} then {set l 1}
           if {$Syspassword($l) == $Userpassword($l)} then {set reply 1} else {set reply 0}
        }
        default {set reply 0}
     }
   }

  if {$reply == 1} then {
     puts stdout "Validated request $request"
  } else {
#    puts stdout "request $request denied"
  }
  return $reply
}

proc make-frame {} {

   :frame [self frame-name] :label "[self frame-title]" :icon-name "[self icon-title]"  :resize F

   :panel P1 :layout L :padx 5 :pady 5 
      :layout V
      :row-gap 5
      :message M1 :label "User Passwords" :bold T
      foreach i {1 2 3 4 5 6 7 8 9} {
        :text PASSWORD$i   :label $i  :w 10 :command click-password :valuex 5c
      }

   :panel P2 :layout R :padx 5 :pady 5
      :layout V
      :row-gap 5
      :message M2 :label "Rules" :bold T
      :galley RULES  :rows 15 :cols 40 

   :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
      :layout H
      :col-gap 5
      :button UPDATE

   :show T

   click-redisplay
}

proc click-password {w v} {
   global Userpassword Syspassword
   set l [string range $w end end]
   set Userpassword($l) $v
   if {$Userpassword($l) != $Syspassword($l)} then \
        {midas-warning "Supplied string does not match the currently required password"}
   click-redisplay
}

proc click-update {} {
   initialise
   click-redisplay
}

proc click-redisplay {} {
    global Userpassword Syspassword rulesdata

    foreach i {1 2 3 4 5 6 7 8 9} {
       :modify PASSWORD$i  :value $Userpassword($i)
       if {$Userpassword($i) == $Syspassword($i)} then {
           :modify PASSWORD$i  :value $Userpassword($i)
       } else {
           :modify PASSWORD$i  :value "[set Userpassword($i)](?)"
       }
    }

    :modify RULES :empty T
    foreach item $rulesdata {
       :modify RULES :strings "[format "%-24s%-8s%-8s" [first $item] [second $item] [third $item]]\n"
    }

}

