#
#     Second (simple!) version of the tape control
#     supports only the MIDAS tape server (via RPC)  [also the GANIL (Exogam) interface]
#
#  


set frame TAPE
set frametitle "Tape Drive Control"
set icontitle "TapeCntl"

#set Version V2
#set SubVersion GANIL
#set SubVersion MIDAS

set Transport SOAP
set Transport RPC

#    some configuration options
set maxdrives 4
set maxtries 300

#    maximum characters in critical names
set driveslen 12

if {$Transport == "RPC"} {
    if {[info exists env(TapeServer)]} then {set resource $env(TapeServer)} else {set resource tape}
    set Tapeserver "/$resource"
    set server $Tapeserver
}

if {$Transport == "SOAP"} {
    if {[info exists env(TapeServer)]} then {set resource $env(TapeServer)} else {set resource localhost:8015}
    set SOAPServerHost [first [split $resource :]]
    set SOAPServerPort [second [split $resource :]]
    loadsource TapeServerClient
}

switch -glob $env(machine) {
    intel -
    *86*  {
       set aliases {0PAT 1PAT 2PAT 3PAT 4PAT 5PAT 6PAT 7PAT}
    }
    default {
       set aliases {TAP0 TAP1 TAP2 TAP3 TAP4 TAP5 TAP6 TAP7}
    }
}

set devstates {unallocated allocated mounting mounted opening open closing executing initialising \
    identifying putting moving twelve thirteen fourteen fifteen "device error"}

set unallocated [lsearch $devstates unallocated]
set allocated [lsearch $devstates allocated]
set mounting [lsearch $devstates mounting]
set mounted [lsearch $devstates mounted]
set opening [lsearch $devstates opening]
set open [lsearch $devstates open]
set closing [lsearch $devstates closing]
set executing [lsearch $devstates executing]
set initialising [lsearch $devstates initialising]
set identifying [lsearch $devstates identifying]
set putting [lsearch $devstates putting]
set moving [lsearch $devstates moving]
#set twelve [lsearch $devstates twelve]
set thirteen [lsearch $devstates thirteen]
set fourteen [lsearch $devstates fourteen]
set fifteen [lsearch $devstates fifteen]
set errorstate [lsearch $devstates "device error"]

set amodelist {0 1 2 8 9 10}
set amodenameslist {"Update" "Read" "Write" "RAW (update)" "RAW (read)" "RAW (write)"}
set labellist {8 2}
set labelnameslist {"Unlabelled" "ANSI"}

set procstatuses { {O.K.} {Client identifier already in use} {Client identifier unknown} {No free resources} \
   {Connect error} {Data management error} {Resource unavailable} {Drive/tape in wrong state} \
   {Invalid capability} {nts_block} {nts_cmd} {Invalid NTS parameter} {nts_nolabel} {EGTS in wrong state} \
   {EGTS already in use} {Invalid EGTS parameter} }


#    DLT device information

set DLTmedia(0x00) "Unknown or media not present"
set DLTmedia(0x81) "Cleaning Tape"
set DLTmedia(0x82) "DLTtape I or DLTtape II"
set DLTmedia(0x83) "DLTtape III"
set DLTmedia(0x84) "DLTtape IIIxt"
set DLTmedia(0x85) "DLTtape IV"
set DLTmedia(0x86) "SDLTtape 1"

set DLTdensity(0x00) "Unknown or media not present"
set DLTdensity(0x1a) "Type IV tape (DLT4000 mode) 20/40 GB"
set DLTdensity(0x1b) "Type IV tape 35/70 GB"
set DLTdensity(0x19) "Type III tape 10GB / IIIXT tape 15 GB (without compression)"
set DLTdensity(0x40) "Type IV tape 40/80 GB"
set DLTdensity(0x41) "Type IV tape 40/80 GB"
set DLTdensity(0x48) "Type SDLT1 tape 110/220 GB"
set DLTdensity(0x49) "Type SDLT1 tape 160/320 GB"
set DLTdensity(0x80) "Type III tape 10GB / IIIXT tape 15 GB (without compression)"
set DLTdensity(0x81) "Type III tape 20GB (DLT2000XT mode) / IIIXT tape 30 GB (with compression)"
set DLTdensity(0x82) "Type IV tape 20 GB (without compression)"
set DLTdensity(0x83) "Type IV tape 40 GB (with compression)"
set DLTdensity(0x84) "Type IV tape 35 GB (without compression)"
set DLTdensity(0x85) "Type IV tape 70 GB (with compression)"
set DLTdensity(0x86) "Type IV tape 40 GB (without compression)"
set DLTdensity(0x87) "Type IV tape 80 GB  (with compression)"
set DLTdensity(0x88) "Type IV tape 40 GB (without compression)"
set DLTdensity(0x89) "Type IV tape 80 GB  (with compression)"
set DLTdensity(0x90) "Type SDLT1 tape 110 GB  (without compression)"
set DLTdensity(0x91) "Type SDLT1 tape 220 GB  (with compression)"
set DLTdensity(0x92) "Type SDLT1 tape 160 GB  (without compression)"
set DLTdensity(0x93) "Type SDLT1 tape 320 GB  (with compression)"


set TapeServerState 0
set Storage 0

set RunNumber 1
set RunPrefix "R"

if {$SubVersion == "MIDAS"} then {
   set ExptName "Expt1"
   set NewName $ExptName
   set OldName $ExptName
} else {
   set Exptname ""
   set NewName ""
   set OldName "Overwrite"
}

#    density is available for setting if needed (determines simulated rate of null device)
set Density 0

#    setup various global arrays   (12 drives is much more than enough)
for {set i 1} {$i <= 12} {incr i} {

#    0 = No;  1 = Yes
    set InUse($i) 0
    set Alias($i) ""
    set Device($i) ""
    set MountOpt($i) 0
}


set aliaslist {}
set usedlist {}

set taperates {0}
set tapelengths {0}
set tapeused {0}
set tapepercent {0}

set fsvisible 0

set m1 "To change the tapes being written to click on the drive \"Allocated?\" button to toggle No/Yes.\n\
        To change a tape or unload (dismount) the tape in a drive click on the drive \"Unload Tape\" button.\n\
        Select the tape write option from the drive \"Load Tape\" menu when the drive has a tape cartridge \
        inserted and is Ready."

set m4 "The Tape Server is busy - please wait"

set m5 "The Tape Server has finished your request"

set merr "An unexpected error has occurred during the execution of your request"

set mabort "Operation aborted at your request"


#    some procedures called from other tasks

proc update-information {} {
    global TSstatus1 TSstatus2
    global taperates tapepercent

    update-all-status

    set taperates [list [lindex $TSstatus1 3]]
    set tapepercent $TSstatus2
}

if {$Transport == "RPC"} {
proc inquire-ganil-status {} {
    global Tapeserver
    return [EG "tape :path '$Tapeserver' ?gstatus"]
}
} else {
proc inquire-ganil-status {} {InquireAcqStatus}
}

proc read-numstreams {} {return 1}

proc read-egstate {} {
    global TapeServerState
    get-server-state
    return $TapeServerState
}

proc read-storagestate {} {
    global Storage
    get-server-state
    return $Storage
}

proc click-egstate {v} {click-state $v; click-redisplay}

proc find-assoc-strings s {
   global TapeServerState Storage
   global maxdrives InUse Device usedlist aliaslist allocated

   get-server-state
   
   if {$Storage == 0} then {

    set drivesinfo [read-drives]
    find-used-drives
    update-configuration

     set reply ""
     for {set i 1} {$i <= $maxdrives} {incr i} {
        if {$InUse($i) != 0} then {
            set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
            set fstatus [query-file-status $alias]
            set state [lindex $fstatus 1]
            set item [lindex $drivesinfo [expr $i-1]]
            if {$state > $allocated} then {append reply "[lindex $item 1]    "}
        }
     }
     if {$reply == ""} then {set reply "No Tapes in Use"}
   } else {
       set reply "Storage is not Enabled"
   }
   return $reply
}

proc find-assoc-strings {args} {
   global TapeServerState Storage
   global maxdrives InUse Device usedlist aliaslist allocated

   get-server-state
   
   if {$Storage == 0} then {

    find-used-drives

    set reply ""
    foreach alias [string range [query-stream-association] 1 end] {append reply "[drives-from-aliases $alias]   "}
    if {$reply == ""} then {set reply "No Tapes in Use"}
   } else {
       set reply "Storage is not Enabled"
   }
   return $reply
}

proc drives-from-aliases al {
   global aliaslist usedlist
   set ll {}
   foreach a $al  {
      set i [lsearch $aliaslist $a]
      if {$i==-1} then {
         lappend ll ????
      } else {
         lappend ll [lindex $usedlist $i]
      }
   }
   return $ll
}

proc Save_Parameters {} {
   global env ExptName RunNumber RunPrefix

   set z [catch {open [file join $env(EXPERIMENTBASE) TapeServerParameters] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   puts $file "$ExptName $RunNumber $RunPrefix"
   close $file
}

proc Restore_Parameters {} {
   global env ExptName RunNumber RunPrefix

   set z [catch {open [file join $env(EXPERIMENTBASE) TapeServerParameters] r} file]
   if {$z != 0} then {
       Save_Parameters
       set z [catch {open [file join $env(EXPERIMENTBASE) TapeServerParameters] r} file]
       if {$z != 0} then {set-footer "$file"; return 1}
   }
   set p [read $file]
   close $file
   set ExptName [first $p]
   set RunNumber [second $p]
   set RunPrefix [third $p]
   if {$RunPrefix == ""} then {set RunPrefix "R"}
   return 0
}

#    basic server access requests

if {$Transport == "RPC"} {
proc read-drives {} {
    global Tapeserver
    EG "tape :path '$Tapeserver' ?drives"
}
} else {
proc read-drives {} {InquireAvailableDevices}
}

if {$Transport == "RPC"} {
proc claim-drive {drive alias} {
    global Tapeserver
    EG "tape :path '$Tapeserver' :alias '$alias' :drive '$drive' !claim"
}
} else {
proc claim-drive {drive alias} {AllocateDevice $alias $drive}
}

if {$Transport == "RPC"} {
proc free-drive {alias} {
    global Tapeserver
    EG "tape :path '$Tapeserver' :alias '$alias' !free"
}
} else {
proc free-device {alias} {DeallocateDevice $alias}
}

proc get-server-state {} {
    global Transport
    global TapeServerState Storage
if {$Transport == "RPC"} {
    global Tapeserver
    set TapeServerState [expr [EG "tape :path '$Tapeserver' ?state"] - 1]
} else {
    set TapeServerState [expr [InquireTapeServerState] - 1]
}
    if {$TapeServerState == 0} then {return}
    if {$TapeServerState == 1} then {
        set Storage 0
    } else {
        set Storage 1
        set TapeServerState 1
    }
}

proc set-server-state {v} {
    global Transport
    global TapeServerState Storage
    set TapeServerState $v
    if {$TapeServerState == 0} then {
        set state 1
    } else {
        set state [expr $TapeServerState + $Storage + 1]
    }
if {$Transport == "RPC"} {
    global Tapeserver
    EG "tape :path '$Tapeserver' :state $state !state"
} else {
    SetTapeServerState $state
}

    get-server-state

   if {$TapeServerState == 0} then {
       insert-log "tape: [date] server state now STOPPED"
       set-footer "Server state now STOPPED"
   } else {
       if {$Storage == 0} then {
           insert-log "tape: [date] server state now GOING"
           set-footer "Server state now GOING"
       } else {
           insert-log "tape: [date] server state now GOING + NOSTORAGE"
           set-footer "Server state now GOING + NOSTORAGE"
       }
   }
}

proc do_pause {} {
    global Transport
    if {$Transport == "RPC"} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :state 8 !state"
    } else {
        SetTapeServerState 8
    }
}

proc do_resume {} {
    global Transport
    if {$Transport == "RPC"} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :state 9 !state"
    } else {
        SetTapeServerState 9
    }
}

proc do_increment_run {} {
   do_pause
   click-state 0
   click-state 1
   do_resume
}

if {$Transport == "RPC"} {
proc query-file-status {alias} {
    global Tapeserver
    EG "tape :path '$Tapeserver' :alias '$alias' ?fstatus"
}
} else {
proc query-file-status {alias} {InquireStreamStatus $alias}
}

proc query-device-status alias {
        global Transport Options
        global DLTsense DLTinquiry DLTmode DriveInfo

if {$Transport == "RPC"} {
        global Tapeserver
        set DriveInfo [EG "tape :path '$Tapeserver' :alias '$alias' ?dstatus"]
} else {
        set DriveInfo [InquireDeviceStatus $alias]
}

        if {[first $DriveInfo] == "file"} {
            load_Options
            if {[info exists Options(tape.getdiscinfo)]} {
                set reply [eval $Options(tape.getdiscinfo)]
                if {[first $reply] == "Filesystem"} {  
                    set DriveInfo [lreplace $DriveInfo 1 1 [lindex $reply 8]]   ;# disc size
                    set DriveInfo [lreplace $DriveInfo 2 2 [lindex $reply 10]]  ;# disc remaining
                }
            }
        }

        set dtype [string range [lindex $DriveInfo 0] 0 2]
        if {[string toupper $dtype] != "DLT"} then {return $DriveInfo}
    
        set DLTsense [lrange $DriveInfo 9 38]
	set DLTinquiry [lrange $DriveInfo 69 124]
	set DLTmode [lrange $DriveInfo 45 56]

#    look for known drive models
   
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "44 4c 54 32 30 30 30 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "DLT2000"]}
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "44 4c 54 34 30 30 30 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "DLT4000"]}
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "44 4c 54 37 30 30 30 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "DLT7000"]}
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "44 4c 54 38 30 30 30 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "DLT8000"]}
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "53 44 4c 54 33 32 30 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "SDLT320"]}
        if {[string compare -length 24 [string range $DLTinquiry 24 47] "53 75 70 65 72 44 4c 54 31 20 "] == 0} \
               then {set DriveInfo [lreplace $DriveInfo 0 0 "SDLT220"]}

        return $DriveInfo
}

#    Note - It seems that DLTs do not always follow the documentation
#    The DLT 4000 at ANU do not have the tape remaining in the documented place
proc get_tape_left {} {
    global DLTsense
    return  [expr 0x[lindex $DLTsense 25][lindex $DLTsense 26][lindex $DLTsense 27][lindex $DLTsense 28]]
}

if {$Transport == "RPC"} {
proc query-stream-state {s} {
    global EGallow Tapeserver
    set EGallow 0x40008
    set state [EG "tape :path '$Tapeserver' :stream $s ?strstate"]
    set EGallow -1
    if {[llength $state] == 1} then {set state [list 0 0 0 0]}
    return $state
}
} else {
proc query-stream-state {s} {
    set state [InquireStreamState $s]
    if {[llength $state] == 1} then {set state [list 0 0 0 0]}
    return $state
}
}

if {$Transport == "RPC"} {
proc set-stream-association {idlist} {
        global Tapeserver

        set i 0
        EG "tape !clear"

        foreach item $idlist {
            EG "tape :idnum $i :assid '$item'"
            incr i
        }

#    send the assocation to the server - Only stream 1 and mode duplicated are used

        EG "tape :path '$Tapeserver' :stream 1 :smode 2 !stream"
}
} else {
proc set-stream-association {idlist} {AssociateStream 1 $idlist}
}

if {$Transport == "RPC"} {
proc query-stream-association {} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :stream 1 ?stream"
}
} else {
proc query-stream-association {} {InquireStreamAssociation 1}
}

if {$Transport == "RPC"} {
proc open-file {alias name} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :alias '$alias' :amode 2 :labels 2 :rlen 0 :blen 0 :file '$name' !open"
}
} else {
proc open-file {alias name} {OpenFile $alias 2 2 0 0 $name}
}

if {$Transport == "RPC"} {
proc close-file {alias} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :alias '$alias' !close"
}
} else {
proc close-file {alias} {CloseFile $alias}
}

if {$Transport == "RPC"} {
proc identify-volume {alias} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :alias '$alias' !idvol"
}
} else {
proc identify-volume {alias} {IdentifyVolume $alias}
}

if {$Transport == "RPC"} {
proc dismount-volume {alias} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :alias '$alias' !disvol"
}
} else {
proc dismount-volume {alias} {DismountVolume $alias}
}

if {$Transport == "RPC"} {
proc mount-volume {alias volume} {
        global Tapeserver
        EG "tape :path '$Tapeserver' :alias '$alias' :volume '$volume' !mntvol"
}
} else {
proc mount-volume {alias volume} {MountVolume $alias $volume}
}

if {$Transport == "RPC"} {
proc initialise-volume {alias newname oldname} {
        global Tapeserver Density
        EG "tape :path '$Tapeserver' :alias '$alias' :volume '$newname' :labels 2 :density $Density :oldvol '$oldname' !invol"
}
} else {
proc initialise-volume {alias newname oldname} {InitialiseVolume $alias 2 0 $oldname}
}


#    Two new procedures added mainly for GANIL and Exogam which use procedures 32 and 33

if {$Transport == "RPC"} {
proc get-run-number {} {
    global Tapeserver

    set State [list 0 0 0 0 0]
    catch {set State [EG "tape :path '$Tapeserver' ?gstatus"]}

    set acq [lindex $State 0]
    set store [lindex $State 1]
    set replay [lindex $State 2]
    set simul [lindex $State 3]
    set run [lindex $State 4]

    return $run
}
} else {
proc get-run-number {} {

    set State [InquireAcqStatus]

    set acq [lindex $State 0]
    set store [lindex $State 1]
    set replay [lindex $State 2]
    set simul [lindex $State 3]
    set run [lindex $State 4]

    return $run
}
}

if {$Transport == "RPC"} {
proc set-run-number {n} {
    global Tapeserver
    if {$n <= 0} then {return}
    catch {EG "tape :path '$Tapeserver' :run $n !run"}
}
} else {
proc set-run-number {n} {
    if {$n <= 0} then {return}
    SetRunNumber $n
}
}

if {$Transport == "RPC"} {
proc find-used-drives {} {
   global  Tapeserver EGallow 
   global  aliases aliaslist usedlist
   set aliaslist {}
   set usedlist {}
   set EGallow 0x4000f
   foreach alias $aliases {
      set m [EG "tape :path '$Tapeserver' :alias '$alias' ?fstatus"]
      if {[llength $m] > 1} then {
         lappend aliaslist $alias
         lappend usedlist [lindex $m 4]
      }
   }
   set EGallow -1
}
} else {
proc find-used-drives {} {
   global  aliases aliaslist usedlist
   set aliaslist {}
   set usedlist {}
   foreach alias $aliases {
      set m [InquireStreamStatus $alias]
      if {[llength $m] > 1} then {
         lappend aliaslist $alias
         lappend usedlist [lindex $m 4]
      }
   }
}
}

proc find-free-alias {} {
   global  aliases aliaslist
   foreach alias $aliases {
      if {[lsearch $aliaslist $alias] == -1} then {return $alias}
   }
}


#    wait for state s2 to be reached by device "alias"
#    state s1 is a permitted intermediate state - anything else is an error
#    for exactly if s2 is open then s1 would be openning
proc wait-for {alias s1 s2} {
    global maxtries devstates

    for {set i 1} {$i <= $maxtries} {incr i} {
        set fstatus [query-file-status $alias]
        set state [lindex $fstatus 1]
        if {$state == $s2} then {return 0}
        if {$state != $s1} then {set i $maxtries}

        after 1000
        update idletasks
    }

#    giving up

    set drive [drives-from-aliases $alias]
    midas-warning "The drive $drive will not move to state [lindex $devstates $s2]\n\
                   Current state is [lindex $devstates $state]"

    return $maxtries
}

proc update-configuration {} {
    global maxdrives
    global InUse Device Tape

#    obtain information on the available drives

    set drivesinfo [read-drives]

#    update the number of drives and redraw the window if needed

    if {$maxdrives != [llength $drivesinfo]} {
        set maxdrives [llength $drivesinfo]
        make-frame
    }

    find-used-drives

#    now update the drive information in the window

    for {set i 1} {$i <= $maxdrives} {incr i} {
        set item [lindex $drivesinfo [expr $i-1]]
        :modify DRIVE$i :label [lindex $item 1]
        set InUse($i)  [lindex $item 0]
        set Device($i) [lindex $item 1]

        if {[lindex $item 0] == "0"} then {
            :modify INUSE$i :value 0
        } else {
            :modify INUSE$i :value 1
        }
        set InUse($i) [lindex $item 0]
    }
}

proc update-widgets {} {
    global SubVersion maxdrives TapeServerState
    global aliaslist usedlist Device mounted opened

if {$SubVersion == "MIDAS"} then {
    set loadstate "opened"
} else {
    set loadstate "mounted"
}

    get-server-state

    if {$TapeServerState != 0} then {

#    GOING

        :modify RUN :inactive T
        :modify STORAGE :inactive T
        for {set i 1} {$i <= $maxdrives} {incr i} {
            :modify UNLOAD$i :inactive T
            :modify LOAD$i   :inactive T
            :modify INUSE$i  :inactive T
        }
     } else {

#    STOPPED

        set drivesinfo [read-drives]

        :modify RUN :inactive F
        :modify STORAGE :inactive F
        for {set i 1} {$i <= $maxdrives} {incr i} {
            set item [lindex $drivesinfo [expr $i-1]]
            :modify INUSE$i  :inactive F
            if {[lindex $item 0] == "0"} then {
                :modify UNLOAD$i :inactive T
                :modify LOAD$i :inactive T
            } else {
                :modify UNLOAD$i :inactive F
                set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
                set fstatus [query-file-status $alias]
                if {[lindex $fstatus 1] < $loadstate} then {
                    :modify LOAD$i :inactive F
                } else {
                    :modify LOAD$i :inactive T
                }
            }
        }
    }
}

proc update-all-status {} {
    global maxdrives aliaslist usedlist
    global devstates allocated
    global InUse Device
    global TSstatus1 TSstatus2

    set TSstatus1 0
    set TSstatus2 0
    set percent 0
    set blklen 16

    if {[winfo ismapped .]} then {:modify PERCENT :value 0}
    for {set i 1} {$i <= $maxdrives} {incr i} {
        if {$InUse($i) == 0} then {
            if {[winfo ismapped .]} then {:modify STATUS$i :label "Not In Use"}
        } else {
            set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
            set fstatus [query-file-status $alias]
            set state [lindex $fstatus 1]
            if {[winfo ismapped .]} then {:modify STATUS$i :label [lindex $devstates $state]}
            if {$state > $allocated} then {
                set st [query-device-status $alias]
                set length [second $st]
                set used [expr [second $st]-[third $st]]
                if {$length > 0} then {
                    set percent [expr ($used*100.0)/$length]
                    set blklen [expr [lindex $fstatus 10] / 1024]
                } else {
                    set percent 0
                }
                if {[winfo ismapped .]} then {:modify PERCENT :value [string range $percent 0 3]}
            }
        }
    }
    set TSstatus2 $percent

    set status [query-stream-state 1]
    if {[winfo ismapped .]} then {
        :modify BLOCKS :value [lindex $status 1]
        :modify KBYTES :value [lindex $status 2]
        :modify RATE   :value [lindex $status 3]
    }
    set TSstatus1 $status
}

proc update-status {i alias} {
    global devstates

    set fstatus [query-file-status $alias]
    :modify STATUS$i :label [lindex $devstates [lindex $fstatus 1]]

    set ps 0x[format %x [first $fstatus]]
    set ss [second $fstatus]
    set info [string range [third $fstatus] 0 [expr [string first ( [third $fstatus]] - 1]]

    if {$ps > 16} then {

#    returns from new MIDAS server

        if {$ps == [expr 0xf070001] || $ps == [expr 0xf090001]} then {set ps 0}
    }

    if {$ps != 0} then {
        midas-warning "Operation failed: Error $ps: $info\nThe current operation will be aborted."
    }
    return $ps
}


proc click-drive {n} {
   global Device InUse aliaslist usedlist allocated
   global devstates procstatuses amodelist amodenameslist labellist labelnameslist
   global CurrentAlias

     if {$InUse($n) == 0} then {return}

     set alias [lindex $aliaslist [lsearch $usedlist $Device($n)]]
     set CurrentAlias $alias
     make-fsframe

     :modify [self frame-name].fs :label "Status for $Device($n)"

     set st [query-file-status $alias]
     set status [first $st]
     set procstatus [lindex $procstatuses $status]
     if {$procstatus != ""} then {
        :modify FS-LAST   :value "$procstatus"
     } else {
        :modify FS-LAST   :value [format "%s (%s)" 0x[format %x $status] $status]
     }
     :modify FS-STATE   :value [lindex $devstates [second $st]]
     :modify FS-INFO    :value [third $st]
     :modify FS-DEVICE  :value [fourth $st]
     :modify FS-RDEVICE :value [lindex $st 4]
     :modify FS-VOLUME  :value [lindex $st 5]
     :modify FS-FILE    :value [first [split [lindex $st 6] /]]
     set v [lsearch $amodelist [lindex $st 7]]
     :modify FS-MODE    :value [lindex $amodenameslist $v]
     set v [lsearch $labellist [lindex $st 8]]
     :modify FS-LABELS  :value [lindex $labelnameslist $v]
     :modify FS-BLOCK   :value [lindex $st 15]
     :modify FS-BYTE    :value [lindex $st 16]
     :modify FS-RATE    :value [lindex $st 17]

     if {[second $st] > $allocated} then {
         set st [query-device-status $alias]
     } else {
         set st [list Unavailable 0 0]
         :modify FS-INFO :value "Device information is unavailable until a tape is loaded"
     }
     :modify DS-TYPE :value  [first $st]
     :modify DS-TLEN :value  [second $st]
     :modify DS-USED :value  [expr [second $st]-[third $st]]
     :modify DS-REM  :value  [third $st]

     if {[string range [first $st] 0 2] == "DLT" || [string range [first $st] 0 3] == "SDLT" } then {
         :modify DS-MORE :inactive F
     } else {
         :modify DS-MORE :inactive T
     }
}

proc show-DLT-info {args} {global CurrentAlias; show_DLT_sense $CurrentAlias}
    

proc click-unload {n} {
   global Device aliaslist usedlist TapeServerState
   global mounted allocated closing

    get-server-state
    if {$TapeServerState != 0} then {midas-warning "You cannot peform this operation while the tape server is GOING.\nYou must STOP first"; return}

    set alias [lindex $aliaslist [lsearch $usedlist $Device($n)]]
    set fstatus [query-file-status $alias]
    set state [lindex $fstatus 1]
 
    if {$state > $mounted} then {

#     state is after mounted - probably open or error  - so try close
#     in any error accept whatever happens

       close-file $alias
       if {[update-status $n $alias] == 0} then {

           wait-for $alias $closing $mounted
           update-status $n $alias
       }
    }

    if {$state > $allocated} then {

#    state is after allocated - should now be mounted (or error)

       dismount-volume $alias
       if {[update-status $n $alias] != 0} then {return}

       wait-for $alias $allocated $allocated
       if {[update-status $n $alias] != 0} then {return}
    }
}

proc changeconfiguration {} {
    global InUse usedlist aliaslist maxdrives TapeServerState

#    update the tape configuration in use by the server
#    if the state (claimed or not claimed) of a drive is different from the InUse flag
#    perform the operation to make them the same

#    obtain information on the available drives

    set drivesinfo [read-drives]

    for {set i 1} {$i <= $maxdrives} {incr i} {
        set item [lindex $drivesinfo [expr $i-1]]
        set CurrentInUse [lindex $item 0]
        set m [lindex $item 1]

        if {$CurrentInUse != $InUse($i)} then {

#    this drive is not currently in the desired state

            if {$InUse($i) == 0} then {

#    desired state is "not claimed" (currently claimed) so free
#    We may first have to close any open file and dismount the tape

                click-unload $i

#    now free
 
                set alias [lindex $aliaslist [lsearch $usedlist $m]]
                free-drive $alias

                :modify STATUS$i :label "Not In Use"

                set-footer "Freed $m"
                insert-log "tape: [date] drive $m freed"
            } else {

#    desired state is "claimed" (currently free) so claim

                find-used-drives
                set alias [find-free-alias]
                claim-drive $m $alias
                if {[update-status $i $alias] != 0} then {return}

                set-footer "Claimed $m as $alias"
                insert-log "tape: [date] drive $m claimed"
            }
       }
    }

    find-used-drives

#BR. 6 aug 2001
#    and update the association 

#    set-stream-association $aliaslist

#    set-footer "Tape configuration changed"
}

proc click-load {i v} {
    global SubVersion maxdrives InUse Device MountOpt aliaslist usedlist TapeServerState
    global allocated mounted open initialising identifying
    global NewName OldName
    global m4 m5 merr mabort

    get-server-state
    if {$TapeServerState != 0} then {midas-warning "You cannot peform this operation while the tape server is GOING.\nYou must STOP first"; return}

    if {$InUse($i) == 0} then {return}

#BR. 6 aug 2001
#     check that all drives already mounted to set correct association

     set Ready ""
     set NumReady 0
     set Text ""

     for {set i1 1} {$i1 <= $maxdrives} {incr i1} {

        if {$InUse($i1) != 0} then {

           set alias [lindex $aliaslist [lsearch $usedlist $Device($i1)]]
           set fstatus [query-file-status $alias]
           set state [lindex $fstatus 1]

           if {$state == $mounted} then  {
               if {$NumReady > 0} then {append Ready " "}
               incr NumReady
               append Ready $alias
           }
        }
     }

           set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
           set fstatus [query-file-status $alias]
           set state [lindex $fstatus 1]

#    check drive is in the correct state

           set s 0
           if {$state == $allocated} then {set s 1}
           if {$SubVersion == "MIDAS"} then {
                if {$state == $mounted && $v == 2} then {set s 1}
           }
           if {$s == 0} then {midas-warning "The drive $Device($i) is in the wrong state for this operation"; return}


#    and update the association 
               if {$NumReady > 0} then {append Ready " "}
               append Ready $alias
               set-stream-association $Ready
               set-footer "Tape configuration changed"
               after 2000 clear-footer

               if {$v != 1} then {

#    option is either new tape or to overwrite old tape  -  perform initialise

#BR. 23 aug 2001 
#    don't get-volume-name in GANIL SubVersion
                   if { $SubVersion != "GANIL" } then {
                      set r [get-volume-name $i $v]
                      if {$r > 0} then {set-footer $mabort; return}
                   } 

                   make_dialog_panel $m4

                   if {$v == 0} then {
                        initialise-volume $alias $NewName ""
                   } else {
                        initialise-volume $alias $NewName $OldName
                   }

                   if {[update-status $i $alias] != 0} then {return}
 
                   set r [wait-for $alias $initialising $allocated]
                   if {[update-status $i $alias] != 0} then {return}

                   if {$r > 0} then {set-footer $merr; return}

                   set-footer "Initialised tape on $Device($i) as $NewName"
                   insert-log "tape: [date] tape initialised on drive $Device($i) as $NewName"

              } else {
                  if {[string range $Device($i) 0 9] == "/dev/file/" || [string range $Device($i) 0 4] == "file/"} then {
                      set r [get-volume-name $i $v]
                      if {$r > 0} then {set-footer $mabort; return}
                  }
              }

#    mount tape on this drive

             if {[string range $Device($i) 0 9] == "/dev/file/" || [string range $Device($i) 0 4] == "file/"} then {

                make_dialog_panel $m4

                mount-volume $alias $NewName
                if {[update-status $i $alias] != 0} then {return}

                set r [wait-for $alias $mounting $mounted]
                if {[update-status $i $alias] != 0} then {return}

                if {$r > 0} then {set-footer $merr; return}

             } else {

                make_dialog_panel $m4

                identify-volume $alias
                if {[update-status $i $alias] != 0} then {return}

                set r [wait-for $alias $identifying $mounted]
                if {[update-status $i $alias] != 0} then {return}

                if {$r > 0} then {set-footer $merr; return}
             }

                make_dialog_panel $m5

                set-footer "Mounted tape on $Device($i)"
                insert-log "tape: [date] tape mounted on drive $Device($i)"
}

proc click-state {v} {
    global SubVersion TapeServerState Storage RunNumber RunPrefix m1 m4
    global maxdrives InUse Device aliaslist usedlist open opening closing mounted devstates

#    get the current state to ensure we are up to date

    get-server-state

    if {$v == 1 && $v == $TapeServerState} then {

#    already in the required state 

        set-footer "Server state already [lindex [list STOPPED GOING] $TapeServerState]"
        return
    }

    if {$Storage == 1} then {
        set-server-state $v
        return
    }

#    set the Tape Server state:  v=0 STOP; v=1 GO

   if {$v == 1} then {

#    attempt to GO - check that all drives are in the correct (mounted) state

     set Ready ""
     set NumReady 0
     set Text ""

     for {set i 1} {$i <= $maxdrives} {incr i} {

        if {$InUse($i) != 0} then {

           set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
           set fstatus [query-file-status $alias]
           set state [lindex $fstatus 1]

           if {$state != $mounted} then {
               append Text "The tape on drive $Device($i) is in state [lindex $devstates $state] and is not ready for data acquisition\n"
           } else {
               if {$NumReady > 0} then {append Ready " "}
               incr NumReady
               append Ready $alias
           }
        }
     }

     if {$Text != "" || $NumReady == 0} then {
         if {$NumReady == 0} then {
             append Text "\nNo tapes are ready for data acquisition.\nYou may continue but no data will be written to tape.\n\
                 \nDo you wish to continue?"
         } else {
             append Text "\nSome of your tapes are not ready for data acquisition.\nIf you continue no data will be written to these tapes.\n\
                 \nDo you wish to continue?"
         }
         if {[midas-confirmation $Text] == 0} then {return}
     }

#    and update the association 

     set-stream-association $Ready
     set-footer "Tape configuration changed"
     after 2000 clear-footer

#    now open the files

     for {set i 1} {$i <= $maxdrives} {incr i} {

        if {$InUse($i) != 0} then {

           set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
           set fstatus [query-file-status $alias]
           set state [lindex $fstatus 1]

           if {$state == $mounted} then {

              make_dialog_panel $m4

              if {[string range $Device($i) 0 9] == "/dev/file/" || [string range $Device($i) 0 4] == "file/"} then {
                  set name $RunPrefix$RunNumber
              } else {
                  set name [uppercase $RunPrefix]$RunNumber
              }
              open-file $alias $name
              if {[update-status $i $alias] != 0} then {return}
 
              wait-for $alias $opening $open
              if {[update-status $i $alias] != 0} then {return}

if {$SubVersion == "MIDAS"} then {
              set-footer "File $name opened on $Device($i)"
              insert-log "tape: [date] file $name opened on drive $Device($i)"
} else {
              set-footer "File opened on $Device($i)"
              insert-log "tape: [date] file opened on drive $Device($i)"
}
           }
        }
      }
      set-server-state $v

   } else {

#    attempt to STOP

      set-server-state $v

#    close all files

     for {set i 1} {$i <= $maxdrives} {incr i} {

        if {$InUse($i) != 0} then {

           set alias [lindex $aliaslist [lsearch $usedlist $Device($i)]]
           set fstatus [query-file-status $alias]
           set state [lindex $fstatus 1]

           if {$state > $mounted} then {

              make_dialog_panel $m4

              close-file $alias
              if {[update-status $i $alias] != 0} then {return}

              wait-for $alias $closing $mounted
              if {[update-status $i $alias] != 0} then {return}

              set-footer "File closed on $Device($i):  [lindex $fstatus 15] blocks written"
              insert-log "tape: [date] file closed on drive $Device($i):  [lindex $fstatus 15] blocks written"
           }
        }
     }

#      STOP complete

if {$SubVersion == "MIDAS"} then {

#    for the MIDAS server increment the run number

      set RunNumber [get-run-number]
      incr RunNumber
      click-run $RunNumber
}

   }
}

proc click-run {v} {
  global SubVersion TapeServerState RunNumber

    get-server-state
    if {$TapeServerState != 0} then {midas-warning "You cannot peform this operation while the tape server is GOING.\nYou must STOP first"; return}

    set-run-number $v
    set RunNumber [get-run-number]

    if {$SubVersion == "MIDAS"} then {Save_Parameters}

    insert-log "tape: [date] Run Number now set to $RunNumber"
}

proc click-expt {v} {
  global ExptName SubVersion

    set ExptName [string range $v 0 5]
    if {$SubVersion == "MIDAS"} then {Save_Parameters}
}

proc click-runprefix {v} {
  global RunPrefix SubVersion

    set RunPrefix [string range $v 0 13]
    if {$SubVersion == "MIDAS"} then {Save_Parameters}
}

proc click-inuse {i v} {
    global TapeServerState InUse

    get-server-state
    if {$TapeServerState != 0} then {
        midas-warning "You cannot peform this operation while the tape server is GOING.\nYou must STOP first"
    } else {
        update-configuration
        set InUse($i) $v
        changeconfiguration
    }
}

proc click-redisplay {args} {
    global SubVersion TapeServerState Storage RunNumber ExptName RunPrefix m1
    global EGreport 

    set EGreport -1
    set-busy

   find-used-drives
   update-configuration

   get-server-state

   update-all-status

   if {$SubVersion == "MIDAS"} then {
      set i [Restore_Parameters]
      if {$TapeServerState == 0} then {set-run-number $RunNumber}
   } else {
      set i 0
   }

   set RunNumber [get-run-number]
   :modify RUN :inactive F :value $RunNumber
   if {$SubVersion == "MIDAS"} then {
       :modify EXPT :value $ExptName
       :modify RUNPREFIX :value $RunPrefix
   }

   if {$i != 0} then {Save_Parameters}

   update-widgets

   :modify STATE   :value $TapeServerState
   :modify STORAGE :value $Storage

   make_dialog_panel $m1

   clear-busy
   set EGreport 0
}

proc click-volume {w v} {
    global NewName OldName
    if {$w == "OLDVOL"} then {
        set OldName [uppercase $v]
    } else {
        set NewName [uppercase [string range $v 0 5]]
    }
}

proc click-done {v} {global Done; set Done $v}

proc get-volume-name {index mode} {
   global OldName NewName ExptName Device Done

   empty_dialog_panel

   :layout H; :row-gap 5; :col-gap 10

   if {$mode == 0} then {
        :message D1 :label "You have requested to initialise a new tape on $Device($index)\n\
                            You may supply a volume name for the tape or accept the default supplied.\n\
                            Then click on the Continue button or click on the Cancel button to abort" :bold T
   }
   if {$mode == 1} then {
        :message D1 :label "You have requested to mount a tape on $Device($index)\n\
                            You may supply a volume name for the tape or accept the default supplied.\n\
                            Then click on the Continue button or click on the Cancel button to abort" :bold T
   }
   if {$mode == 2} then {
        :message D1 :label "You have requested to re-initialise the tape on $Device($index)\n\
                            This will cause all data on the tape to be lost.\n\
                            To continue you must enter the EXISTING volume name on the tape which will be checked.\n\
                            Then click on the Continue button or click on the Cancel button to abort" :bold T
   }

   set NewName [uppercase $ExptName]
   set OldName ""

   :next-row T
   if {$mode != 1} then {
       :text NEWVOL :label "NEW Volume name:"      :w 8  :command click-volume  :value $NewName
   } else {
       :text NEWVOL :label "Volume name:"          :w 8  :command click-volume  :value $NewName
   }
   if {$mode == 2} then {
        :text OLDVOL :label "EXISTING Volume name:" :w 8  :command click-volume
   }
   :button CONTINUE :command "click-done 0"
   :button CANCEL   :command "click-done 1"

   tkwait variable Done

   return $Done
}

proc click arg {
   global Storage DS_DRIVE

   set f [first $arg]
   set c [second $arg]
   set w [third $arg]
   set v [fourth $arg]
   if {$c==":quit"} then {click-quit $f; return}
   clear-footer 

   catch {
      switch -glob $w {
         LOAD*    {click-load [string range $w 4 end] [fifth $arg]}
         UNLOAD*  {click-unload [string range $w 6 end]}
         INUSE*   {click-inuse [string range $w 5 end] $v}
         DRIVE*   {set DS_DRIVE [string range $w 5 end]; click-drive $DS_DRIVE}
         DS-UPDATE {click-drive $DS_DRIVE}
         STORAGE  {set Storage $v}
         REDISPLAY  {}
         default {
            set p "click-[lowercase $w]"
            if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
         }
      }
   }
   click-redisplay
}

proc start args {
   global started alias drive curdrv freelist

   if {$started != 0} then {open-frame} else {make-frame; set started 1}
   click-redisplay
}

proc frame-quit win  {
   global fsvisible
   catch {wm iconify [winfo toplevel $win]}
   wm withdraw [winfo toplevel $win]
   if {$win == "[self frame-name].fs"} then {set fsvisible 0}
}

proc empty_dialog_panel {} {:modify DIALOG :empty T}

proc make_dialog_panel {messages} {
   global TapeServerState

   empty_dialog_panel
   update

   :layout V; :row-gap 0

   if {$TapeServerState != 0} then {
      :message D1 :label "The Tape Server is in the GOING status. In this state you can only view the statistics information.\n\n\
                          STOP the Tape Server first if you wish to changes tapes or change the tape configuration."  
   } else {
      :message D1 :label "The Tape Server is in the STOPPED status." 
      :row-gap -5
      :message D2 :font "times 10"  :label $messages  
   }
   :message D99  :label "For more information click on the \"Help\" button or use the F1 key." 
}

proc make-frame {} {
   global env SubVersion
   global maxdrives driveslen

   :frame [self frame-name] :label [self frame-title] :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

   :container C1 :layout T

    :panel P1 :layout L :padx 10 :pady 10
       :layout V
       :row-gap 10; :col-gap 10
       :message M1 :label "Available Drives"

        for {set i 1} {$i <= $maxdrives} {incr i} {
if {$SubVersion == "MIDAS"} then {
           :button DRIVE$i :label ""  :w $driveslen
} else {
           :message DRIVE$i  :label "" :w 12 :bold T
}
        }

    :panel P2 :layout L :padx 10 :pady 10
       :layout V
       :row-gap 10; :col-gap 10
       :message M2 :label "Allocated?"

        for {set i 1} {$i <= $maxdrives} {incr i} {
           :optionbutton INUSE$i :strings [list No Yes]  :w 4
        }

    :panel P3 :layout L :padx 10 :pady 10
       :layout V
       :row-gap 10; :col-gap 10
       :message M3 :label "Status" :w 12

        for {set i 1} {$i <= $maxdrives} {incr i} {
           :message STATUS$i :label "Unknown" :w 12 :bold T
        }

    :panel P4 :layout L :padx 10 :pady 10
       :layout V
       :row-gap 10; :col-gap 10
       :message M4 :label ""

        for {set i 1} {$i <= $maxdrives} {incr i} {
           :button UNLOAD$i :label "Unload Tape"
        }

    :panel P5 :layout L :padx 10 :pady 10
       :layout V
       :row-gap 10; :col-gap 10
       :message M5 :label ""

        for {set i 1} {$i <= $maxdrives} {incr i} {
           :menubutton LOAD$i :label "Load Tape" :value 0  \
                :strings [list "Initialise (New tape)" "Mount (Append Data to old tape)" "Re-Initialise (Overwrite Data on old tape)"]
        }

   :container C2 :layout T

    :panel DIALOG :layout T :padx 5 :pady 0 :w 100 :h 120 
       [:path DIALOG] configure -relief raised -bd 1

    :panel STATUS :layout T :padx 10 :pady 10 
        :layout H; :col-gap 10; :row-gap 10
        :text BLOCKS  :label "Blocks written:" :w 8 :read-only T
        :text KBYTES  :label "Kbytes written:" :w 10 :read-only T
        :text RATE    :label Kbytes/sec: :w 5 :read-only T
        :text PERCENT :label "Tape written %" :w 4 :read-only T

    :panel BUTTONS :layout T :padx 10 :pady 0
        :layout H; :col-gap 10; :row-gap 10
if {$SubVersion == "MIDAS"} then {
        :row-gap 5
        :number RUN  :label "Run Number" :w 3 :min 1 :max 999
        :text  RUNPREFIX :label "Run Name Prefix" :w 14
        :text   EXPT :label "Experiment Name" :w 6
        :next-row T
}
        :choice STATE :label "Server state:"  :exclusive T :strings [list Stop Go] :value 0
        :checkbox STORAGE :label "No Storage" :value 0
if {$SubVersion == "GANIL"} then {
        :number RUN  :label "Run Number" :w 6
}
        :next-row T
        :button REDISPLAY
        :col-gap 10
        :button HELP

    :show T
}


proc make-fsframe {} {
    global fsvisible

set offset1 22c
set offset2 14c

   if {$fsvisible == 1} then {raise [self frame-name].fs; return}

  :frame [self frame-name].fs :show-footer T :resize F :transient T

    :panel FS1 :layout L :padx 10 :pady 5
     :layout V; :row-gap 5

        :text FS-LAST    :label "Last procedure status:" :w 36 :read-only T :valuex $offset1
        :text FS-STATE   :label "Current stream state:"  :w 36 :read-only T :valuex $offset1
        :text FS-DEVICE  :label "Requested Device:"      :w 20 :read-only T :valuex $offset1
        :text FS-RDEVICE :label "Allocated Device:"      :w 20 :read-only T :valuex $offset1
        :text FS-VOLUME  :label Volume:                  :w 20 :read-only T :valuex $offset1
        :text FS-FILE    :label File:                    :w 20 :read-only T :valuex $offset1
        :text FS-MODE    :label Mode:                    :w 20 :read-only T :valuex $offset1
        :text FS-LABELS  :label Labels:                  :w 20 :read-only T :valuex $offset1

    :panel FS2 :layout L :padx 10 :pady 5
     :layout V; :row-gap 5; :col-gap 10

        :text DS-TYPE :label "Device Type:" :w 12 :valuex $offset2 :read-only T
        :text DS-TLEN :label "Tape Length:" :w 12 :valuex $offset2 :read-only T
        :text DS-USED :label "Tape Used:"   :w 12 :valuex $offset2 :read-only T
        :text DS-REM  :label "Tape Left:"   :w 12 :valuex $offset2 :read-only T

      :layout H
      :next-row T
        :button DS-MORE :label "More" :command "show-DLT-info"
        :button DS-UPDATE :label "Update"
      :next-row T
      :layout V

        :text FS-BLOCK :label "Blocks written:" :w 12 :read-only T  :valuex $offset2
        :text FS-BYTE  :label "Kbytes written:" :w 12 :read-only T  :valuex $offset2
        :text FS-RATE  :label "Kbytes/sec:"     :w 12 :read-only T  :valuex $offset2

    :panel FS4 :layout B :padx 10 :after FOOTER
     :layout H; :row-gap 5

        :text FS-INFO :label Information: :w 80 :read-only T

     :show T

   set fsvisible 1
}

proc show_DLT_sense {alias} {
  global DLTsense DLTinquiry DLTmode
  global DLTmedia DLTdensity

   query-device-status $alias

  :frame [self frame-name].ds :label "Current Drive Sense Information" :show-footer T :resize F :transient T

    :panel DSEN-PANEL :layout L :padx 10 :pady 10
     :layout V; :row-gap 5

       :galley DSEN-INFO :rows 20 :cols 90

            :modify DSEN-INFO :strings "Sense Information :-\n$DLTsense\n\n"
            set i [expr 0x[lindex $DLTsense 2] &  0x80]
            if {$i == 1} then {:modify DSEN-INFO :strings "File Mark read by last command\n"}
            set i [expr 0x[lindex $DLTsense 2] &  0x40]
            if {$i == 1} then {:modify DSEN-INFO :strings "End of Medium found\n"}
            set i [expr 0x[lindex $DLTsense 2] &  0x20]
            if {$i == 1} then {:modify DSEN-INFO :strings "Incorrect Length Indicator set\n"}
            set i [expr 0x[lindex $DLTsense 0] &  0x80]
            set info "[lindex $DLTsense 3] [lindex $DLTsense 4] [lindex $DLTsense 5] [lindex $DLTsense 6]"
            if {$i == 1} then {:modify DSEN-INFO :strings "Information bytes :- $info\n"}
            :modify DSEN-INFO :strings "Sense Key :- [expr 0x[lindex $DLTsense 2] & 15]\n"
            :modify DSEN-INFO :strings "Additional Sense Code :- [lindex $DLTsense 12]\n"
            :modify DSEN-INFO :strings "Additional Sense Code Qualifier :- [lindex $DLTsense 13]\n"
            :modify DSEN-INFO :strings "Internal Status Code :- 0x[lindex $DLTsense 18]\n"
            set i [expr 0x[lindex $DLTsense 19][lindex $DLTsense 20]]
            :modify DSEN-INFO :strings "Tape Motion (hours) :- $i\n"
            set i [expr 0x[lindex $DLTsense 21][lindex $DLTsense 22][lindex $DLTsense 23][lindex $DLTsense 24]]
            :modify DSEN-INFO :strings "Power On (hours) :- $i\n"
            set i [get_tape_left]
            :modify DSEN-INFO :strings "Tape Remaining (4 KB) :- $i\n"

            :modify DSEN-INFO :strings "\n\nIn the event of repeated drive errors please give this information to your support group\n\n"

            :modify DSEN-INFO :strings "Mode Information :-\n$DLTmode\n\n"
            set i 0x[lindex $DLTmode 1]
            if {[info exists DLTmedia($i)]} then {set m $DLTmedia($i)} else {set m $DLTmedia(0x00)}
            :modify DSEN-INFO :strings "Media Type :- $i: $m\n"
            set i [expr 0x[lindex $DLTmode 2] &  0x80]
            if {$i == 1} then {
               :modify DSEN-INFO :strings "Tape is write protected\n"
            } else {
               :modify DSEN-INFO :strings "Tape is write enabled\n"
            }
            set i 0x[lindex $DLTmode 4]
            if {[info exists DLTdensity($i)]} then {set m $DLTdensity($i)} else {set m $DLTdensity(0x00)}
            :modify DSEN-INFO :strings "Density Code :- $i - $m\n\n\n"

            :modify DSEN-INFO :strings "Inquiry Information :-\n[lrange $DLTinquiry 0 7]\n[lrange $DLTinquiry 8 15]\n[lrange $DLTinquiry 16 31]\n[lrange $DLTinquiry 32 35]\n[lrange $DLTinquiry 36 55]\n"
 
            :modify DSEN-INFO :see 0

    :show T
}



#    The Options are experiment specific and normally used by SYS
#    However the same code as for the session preferences is used which allows for wider use in the future.
#    In this event these procedures may be moved to sys.common.tcl

proc get_Option_file {} {
    global env

#    look for the Option file 
    set pd $env(EXPERIMENTDIR)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

#    load all Options for the current task
proc load_Options {} {
   global env Options

   set lines [get_Option_file]
#   set key [Get_My_Name]
   set key "sys"
   set range [find-keyword $lines $key]

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           set Options([head $item]) [tail $item]
       }
   }
}
